import java.io.*;
import java.util.*;

/** Restaurant customer agent. */
public class CustomerAgent extends Agent {
	private String name;
	private int hungerLevel = 5;		// determines length of meal

	// agent connections
	private MaitreDAgent maitreD;

	// agent state
	private boolean isHungry = false;
	private boolean isWaiting = false;
	private boolean isSeated = false;

	public CustomerAgent(String name) {
		super();
		this.name = name;
		startThread();
	}

	/** hack to establish connection to Maitre'D agent. */
	public void setMaitreD(MaitreDAgent maitreD) {
		this.maitreD = maitreD;
	}

	/** Scheduler.  Determine what action is called for, and do it. */
	boolean respondToStateChange() {
		//notice here, there is unification, just simple predicates.
		//notice also, the simple view of scheduling: Pick first rule that fires
		//We could look at all the premises and see which are Possibilities.
		//Then use some Search algorithm to pick "best" action to execute.
		//This is a complex issue that we don't care about here.
		if (isHungry && !isWaiting  & !isSeated) {
			goToRestaurant();
			return true;
		}
		if (isWaiting && isSeated) {
			startEating();
			return true;
		}
		if (isSeated && !isHungry) {
			leaveTable();
			return true;
		}
		return false;
	}

	// Actions

	private void goToRestaurant() {
		print("Going to retaurant");
		maitreD.msgHelloIWantFood(this);//send him our instance, so he can respond to us
		isWaiting = true;
		stateChanged();
	}

	private void startEating() {
		isWaiting = false;
		print("Eating for " + hungerLevel*1000 + " milliseconds.");
		//This next complicated line creates and starts a timer thread.
		//In it we schedule a deadline of getHungerLevel()*1000 milliseconds.
		//When that time elapses, it will call back to the little timerExpired routine
		//located in the anonymous class created right there inline:
		//		TimerQueue.Callback is an interface that we implement right there inline.
		//Since Java does not all us to pass functions, only objects. So, we use
		//Java syntactic mechanism to create an anonymous inner class that has
		//the public method timerExpired in it.
		TimerQueue.schedule(Deadline.in(getHungerLevel() * 1000),
							new TimerQueue.Callback() {
								public void timerExpired(Object cookie) {
									print("Done eating");
									isHungry = false;
									stateChanged();
								}},
								null);//Pass information to timer using a cookie: here null.
	}

	private void leaveTable() {
		maitreD.msgLeavingTable(this);
		isSeated = false;
		stateChanged();
		becomeHungryInAWhile();//set a timer to make us hungry.
	}

	private void becomeHungryInAWhile() {
		TimerQueue.schedule(Deadline.in(15000),
							new TimerQueue.Callback() {
								public void timerExpired(Object cookie) {
									setHungry();
								}},
								null);
	}

	// Messages

	public void msgSitAtTable() {
		print("Received msgSitAtTable");
		isSeated = true;
		stateChanged();
	}


	// Accessors, etc.

	protected String getName() {
		return name;
	}

	public void setHungry() {
		isHungry = true;
		print("I'm hungry");
		stateChanged();
	}

	public int getHungerLevel() {
		return hungerLevel;
	}

	public void setHungerLevel(int hungerLevel) {
		this.hungerLevel = hungerLevel; //could be a state change. Maybe you don't
										//need to eat until hunger lever is > 5?
	}

	public String toString() {
		return "customer " + getName();
	}

}

