import java.io.*;
import java.util.*;

/** Restaurant Maitre"D Agent */
//We only have 2 types of agents in this prototype. A customer and an agent that
//does all the rest. Rather than calling the other agent a waiter, we called him
//the MaitreDAgent. A MaitreD is the manager of a restaurant who sees that all
//is proceeded as he wishes.
public class MaitreDAgent extends Agent {
	static final int NTABLES = 2;//a global for the number of tables.
		//Notice that we implement waitingCustomers using LinkedList, but type it
		//with List semantics.
	List waitingCustomers = new LinkedList();
	Collection tables;	//note that tables is typed with Collection semantics.
						//Later we will see how it is implemented

	public MaitreDAgent() {
		super();
		// make some tables
		tables = new ArrayList(NTABLES);
		for (int ix = 1; ix <= NTABLES; ix++) {
			tables.add(new Table(ix));//how you add to a collections
		}
		startThread();	//create and start the agent thread.
		//usually a startThread call is not done in the constructor, but...
	}

	/** Scheduler.  Determine what action is called for, and do it. */
	boolean respondToStateChange() {
		/* Think of this next rule as:
		  Does there exist a table and customer,
			so that table is unoccupied and customer is waiting. If so seat him at the table.
			in FOL: Et, Ec ~isOccupied(t) & waitingCustomer(c) => seatCustomer(c,t)
		*/
		for (Iterator iter = tables.iterator(); iter.hasNext(); ) {
			Table table = (Table)iter.next();
			if (!table.isOccupied()) {//trying to UNIFY a table with the ~isOccupied(t) clause
				//next try to UNIFY a customer with waitingCustomer(c)
				if (!waitingCustomers.isEmpty()) {
					//we have matched the premise; invoke the conclusion, with the
					//substitutions found during unification.
					seatCustomer((CustomerAgent)waitingCustomers.get(0), table);//the action
					return true;	//after the action, we return true to the main loop of
									//abstract agent in order to invoke the scheduler again.
				}
			}
		}
		return false;	//we have tried all our rules (in this case only one) and found
						//nothing to do. So return false to main loop of abstract agent
						//and wait.
	}

	// Actions

	private void seatCustomer(CustomerAgent customer, Table table) {
			//Notice how we print "customer" directly. It's toString method will do it.
			//Same with "table"
		print("Seating " + customer + " at " + table);
		table.setOccupant(customer);
		waitingCustomers.remove(customer);
		customer.msgSitAtTable();
		stateChanged();
	}

	void mealOver(Table table) {//Hmmm. Is this an action?
		CustomerAgent cust = table.getOccupant();
		print(cust + " leaving " + table);
		table.setUnoccupied();
		stateChanged();
	}


	// Messages

	public void msgHelloIWantFood(CustomerAgent cust) {
		waitingCustomers.add(cust);
		stateChanged();
	}

	public void msgLeavingTable(CustomerAgent cust) {
		for (Iterator iter = tables.iterator(); iter.hasNext(); ) {
			Table table = (Table)iter.next();
			if (table.getOccupant() == cust) {//will be different with RMI.
				mealOver(table); //Action????????? Should be inline.
			}
		}
	}

	/*
	This private class would be inappropriate if there were waiters and busboys. Both
	should be sharing the same Table data structure. HOWEVER, AGENTS DON'T SHARE DATA.
	We would create a TableAgent which acts as a database agent for the other agents.
	You interface to a database agent as you would to any SQL server, i.e., the messages
	set and get information about the tables (in a polite way).
	Extra credit if you implement such an agent.
	Extra extra credit if you use JDBC and a real data base, say MYSQL.
	*/
	class Table {
		CustomerAgent occupiedBy;
		int tableNumber;

		Table(int tableNumber) {
			this.tableNumber = tableNumber;
		}

		void setOccupant(CustomerAgent cust) {
			occupiedBy = cust;
		}

		void setUnoccupied() {
			occupiedBy = null;
		}

		CustomerAgent getOccupant() {
			return occupiedBy;
		}

		boolean isOccupied() {
			return occupiedBy != null;
		}

		public String toString() {
			return "table " + tableNumber;
		}
	}
}

